#include "moso.h"
#include "common.h"
#include "png2bmp.h"

void setScreen(CMoso* pMe)
{
	setClip(pMe,0,0,pMe->canvas_width,pMe->canvas_height);
}

void setClip(CMoso *pMe,int x, int y, int w, int h)
{
	pMe->clipRect.x = x;
	pMe->clipRect.y = y;
	pMe->clipRect.dx = (w < 0 ? 0 : w);
	pMe->clipRect.dy = (h < 0 ? 0 : h);
}

void loadImage(CMoso *pMe,Image* bm, char* pszBarFile,int resId)
{
	boolean allocated = FALSE;

	// if bm already have a image, just return;
	if( bm->bitmap != NULL || bm->src != NULL ) return;	

	bm->src = PNG2BMP_LoadResData(pMe->a.m_pIShell, pszBarFile, (int16)resId);
	bm->bitmap = CONVERTBMP(bm->src, &bm->info, &allocated);

	if (allocated) 
	{
		FREE(bm->src);
		bm->src = NULL;
	}
}

void freeImage(CMoso *pMe,Image* bm)
{
	if (bm->bitmap) {
		SYSFREE(bm->bitmap);
		bm->bitmap = NULL;
	}
	if (bm->src) {
		FREE(bm->src);
		bm->src = NULL;
	}
}

void drawImage(CMoso *pMe,int resID, int x, int y, int flag)
{
	int	offX,offY;					// 
	int	ox1,ox2,oy1,oy2;			// original rect
	int	cx1,cx2,cy1,cy2;			// clip rect
	int	nx1,nx2,ny1,ny2,ndx,ndy;	// new rect
	Image*	bm = &pMe->imgGame[resID];

	// if bitmap is null, return;
	if( bm->bitmap == NULL ) return;

	// ϴ flag  ġ   .
	// horizontal
	if (flag & GFX_HCENTER) {
		x = x - (bm->info.cx >> 1);
	} else if (flag & GFX_RIGHT) {
		x = x - bm->info.cx;
	}
	// vertical
	if (flag & GFX_VCENTER) {
		y = y - (bm->info.cy >> 1);
	} else if (flag & GFX_BOTTOM) {
		y = y - bm->info.cy;
	}
	//=========================================


	// get original image rect
	ox1 = x;
	oy1 = y;
	ox2 = x + bm->info.cx;
	oy2 = y + bm->info.cy;

	// get clip rect
	cx1 = pMe->clipRect.x;
	cy1 = pMe->clipRect.y;
	cx2 = cx1 + pMe->clipRect.dx;
	cy2 = cy1 + pMe->clipRect.dy;

	// ׸ ϴ  Ŭ  ۿ  ..
	if( cx1 > ox2 || cx2 < ox1 ||
		oy1 > cy2 || oy2 < cy1 )
		return;
	//=========================================

	nx1 = ( cx1 < ox1 ) ? ox1 : cx1;
	ny1 = ( cy1 < oy1 ) ? oy1 : cy1;
	nx2 = ( cx2 < ox2 ) ? cx2 : ox2;	
	ny2 = ( cy2 < oy2 ) ? cy2 : oy2;

	ndx = nx2 - nx1;
	ndy = ny2 - ny1;

	offX = nx1 - ox1;
	offY = ny1 - oy1;

	IDISPLAY_BitBlt(pMe->a.m_pIDisplay,
					nx1, ny1, ndx, ndy,		// clipping area
					bm->bitmap, offX, offY, AEE_RO_MASK);
}

void drawImage2(CMoso *pMe,Image* bm, int x, int y, int flag)
{
	int	offX,offY;					// 
	int	ox1,ox2,oy1,oy2;			// original rect
	int	cx1,cx2,cy1,cy2;			// clip rect
	int	nx1,nx2,ny1,ny2,ndx,ndy;	// new rect

	// if bitmap is null, return;
	if( bm->bitmap == NULL ) return;

	// ϴ flag  ġ   .
	// horizontal
	if (flag & GFX_HCENTER) {
		x = x - (bm->info.cx >> 1);
	} else if (flag & GFX_RIGHT) {
		x = x - bm->info.cx;
	}
	// vertical
	if (flag & GFX_VCENTER) {
		y = y - (bm->info.cy >> 1);
	} else if (flag & GFX_BOTTOM) {
		y = y - bm->info.cy;
	}
	//=========================================


	// get original image rect
	ox1 = x;
	oy1 = y;
	ox2 = x + bm->info.cx;
	oy2 = y + bm->info.cy;

	// get clip rect
	cx1 = pMe->clipRect.x;
	cy1 = pMe->clipRect.y;
	cx2 = cx1 + pMe->clipRect.dx;
	cy2 = cy1 + pMe->clipRect.dy;

	// ׸ ϴ  Ŭ  ۿ  ..
	if( cx1 > ox2 || cx2 < ox1 ||
		oy1 > cy2 || oy2 < cy1 )
		return;
	//=========================================

	nx1 = ( cx1 < ox1 ) ? ox1 : cx1;
	ny1 = ( cy1 < oy1 ) ? oy1 : cy1;
	nx2 = ( cx2 < ox2 ) ? cx2 : ox2;	
	ny2 = ( cy2 < oy2 ) ? cy2 : oy2;

	ndx = nx2 - nx1;
	ndy = ny2 - ny1;

	offX = nx1 - ox1;
	offY = ny1 - oy1;

	IDISPLAY_BitBlt(pMe->a.m_pIDisplay,
					nx1, ny1, ndx, ndy,		// clipping area
					bm->bitmap, offX, offY, AEE_RO_MASK);
}

void setColor(CMoso *pMe,uint8 r,uint8 g,uint8 b)
{
	IDISPLAY_SetColor(pMe->a.m_pIDisplay,CLR_USER_TEXT,MAKE_RGB(r,g,b));
}

void fillRect(CMoso* pMe,int x,int y,int width,int height,RGBVAL rgb)
{
	AEERect rt;
	int x1,dx,y1,dy;

	/*
	// correction region with clipRect
	x1 = ( pMe->clipRect.x > x ) ? pMe->clipRect.x : x;
	dx = ( (pMe->clipRect.x + pMe->clipRect.dx) < (x + width) ) ? (pMe->clipRect.x + pMe->clipRect.dx) : (x + width);
	dx = dx - x1;

	y1 = ( pMe->clipRect.y > y ) ? pMe->clipRect.y : y;	
	dy = ( (pMe->clipRect.y + pMe->clipRect.dy) < (y + height) ) ? (pMe->clipRect.y + pMe->clipRect.dy) : (y + height);
	dy = dy - y1;
	*/
	// correction region with clipRect
	if( pMe->clipRect.x > x ) 
		x1 = pMe->clipRect.x;
	else
		x1 = x;
	if( (pMe->clipRect.x + pMe->clipRect.dx) < (x + width) )
		dx = (pMe->clipRect.x + pMe->clipRect.dx);
	else
		dx = (x + width);
	dx = dx - x1;

	if( pMe->clipRect.y > y ) 
		y1 = pMe->clipRect.y;	
	else
		y1 = y;	
	if( (pMe->clipRect.y + pMe->clipRect.dy) < (y + height) )
		dy = (pMe->clipRect.y + pMe->clipRect.dy);
	else
		dy = (y + height);
	dy = dy - y1;

	if( dx <= 0 || dy <= 0 ) return;	// dx  dy  0    ..

	SETAEERECT(&rt,x1,y1,dx,dy);
	IDISPLAY_FillRect(pMe->a.m_pIDisplay, &rt, rgb);
}

void drawRect(CMoso* pMe,int x,int y,int width,int height,RGBVAL rgb)
{
	AEERect rt;
	int x1,dx,y1,dy;

	// correction region with clipRect
	x1 = ( pMe->clipRect.x > x ) ? pMe->clipRect.x : x;
	dx = ( pMe->clipRect.x + pMe->clipRect.dx < x + width ) ? pMe->clipRect.x + pMe->clipRect.dx : x + width;
	dx -= x1;

	y1 = ( pMe->clipRect.y > y ) ? pMe->clipRect.y : y;	
	dy = ( pMe->clipRect.y + pMe->clipRect.dy < y + height ) ? pMe->clipRect.y + pMe->clipRect.dy : y + height;
	dy -= y1;
	
	if( dx <= 0 || dy <= 0 ) return;	// dx  dy  0    ..

	SETAEERECT(&rt,x1,y1,dx,dy);
	IDISPLAY_DrawRect(pMe->a.m_pIDisplay, &rt, rgb,0,IDF_RECT_FRAME);
}

void drawString(CMoso *pMe,char* str,int x, int y,int nFlag)
{
	int strX1,strX2,strY1,strY2;
	int clipX1,clipX2,clipY1,clipY2;
	int interX1,interX2,interY1,interY2;

	AEERect rt;
	int wsize;

	// convert single byte char to multi byte char.
	MEMSET(pMe->tempWStr,0,100);	
	StrToWStr( str, pMe->tempWStr );	// copy single byte character to multibyte character.
	
	// get text's width.
	wsize = IDISPLAY_MeasureText(pMe->a.m_pIDisplay,pMe->font,pMe->tempWStr);	

	// horizontal
	if	( GFX_HCENTER & nFlag )		x = x - (wsize >> 1);
	else if	( GFX_RIGHT & nFlag )	x = x - wsize;
	// vertical
	if	( GFX_VCENTER & nFlag )		y = y - (pMe->font_h >> 1);
	else if	( GFX_BOTTOM & nFlag )	y = y - pMe->font_h;

   	SETAEERECT(&rt,x,y,wsize,pMe->font_h);

	strX1 = rt.x;	strX2 = rt.x + rt.dx;
	strY1 = rt.y;	strY2 = rt.y + rt.dy;

	clipX1 = pMe->clipRect.x;	clipX2 = pMe->clipRect.x + pMe->clipRect.dx;
	clipY1 = pMe->clipRect.y;	clipY2 = pMe->clipRect.y + pMe->clipRect.dy;

	interX1 = (strX1 > clipX1) ? strX1 : clipX1;
	interX2 = (strX2 < clipX2) ? strX2 : clipX2;

	interY1 = (strY1 > clipY1) ? strY1 : clipY1;
	interY2 = (strY2 < clipY2) ? strY2 : clipY2;

	
	rt.x  = interX1;
	rt.dx = interX2 - interX1;
	rt.y  = interY1;
	rt.dy = interY2 - interY1;


	if( y < rt.y || y > rt.y + rt.dy ) return;
	//    ż..
	//fillRect(pMe,rt.x,rt.y,rt.dx,rt.dy,MAKE_RGB(0,0,255));


	IDISPLAY_DrawText(pMe->a.m_pIDisplay ,pMe->font, 
		pMe->tempWStr,-1,x,y, &rt,IDF_TEXT_TRANSPARENT);	
}

void drawWString(CMoso *pMe,AECHAR* str,int x, int y,int nFlag)
{
	int strX1,strX2,strY1,strY2;
	int clipX1,clipX2,clipY1,clipY2;
	int interX1,interX2,interY1,interY2;

	AEERect rt;
	int wsize;

	// get text's width.
	wsize = IDISPLAY_MeasureText(pMe->a.m_pIDisplay,pMe->font,str);	

	// horizontal
	if	( GFX_HCENTER & nFlag )		x = x - (wsize >> 1);
	else if	( GFX_RIGHT & nFlag )	x = x - wsize;
	// vertical
	if	( GFX_VCENTER & nFlag )		y = y - (pMe->font_h >> 1);
	else if	( GFX_BOTTOM & nFlag )	y = y - pMe->font_h;

   	SETAEERECT(&rt,x,y,wsize,pMe->font_h);

	strX1 = rt.x;	strX2 = rt.x + rt.dx;
	strY1 = rt.y;	strY2 = rt.y + rt.dy;

	clipX1 = pMe->clipRect.x;	clipX2 = pMe->clipRect.x + pMe->clipRect.dx;
	clipY1 = pMe->clipRect.y;	clipY2 = pMe->clipRect.y + pMe->clipRect.dy;

	interX1 = (strX1 > clipX1) ? strX1 : clipX1;
	interX2 = (strX2 < clipX2) ? strX2 : clipX2;

	interY1 = (strY1 > clipY1) ? strY1 : clipY1;
	interY2 = (strY2 < clipY2) ? strY2 : clipY2;

	
	rt.x  = interX1;
	rt.dx = interX2 - interX1;
	rt.y  = interY1;
	rt.dy = interY2 - interY1;


	if( y < rt.y || y > rt.y + rt.dy ) return;
	//    ż..
	//fillRect(pMe,rt.x,rt.y,rt.dx,rt.dy,MAKE_RGB(0,0,255));


	IDISPLAY_DrawText(pMe->a.m_pIDisplay ,pMe->font, 
		str,-1,x,y, &rt,IDF_TEXT_TRANSPARENT);	
}

// copy single byte character to multi byte character.
void StrToWStr(char* src,AECHAR* dst)
{
	while(*src)
	{
		// , ȣ..
		if((unsigned char)*src<128)
		{ 
			*dst=*src; 
			src++;
		} else
		{ 
			MEMCPY( dst, src, sizeof(AECHAR) );
			src+=2;
		}
		dst++;
	}
	*dst=0; 
}

void debugText(CMoso* pMe, const char* text)
{
	AECHAR buffer[64];

	setColor(pMe,0,0,0);
	STR_TO_WSTR((char*) text, buffer, 64);
	IDISPLAY_ClearScreen(pMe->a.m_pIDisplay);
	IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, buffer, -1, 0, 0, 0, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
	IDISPLAY_Update(pMe->a.m_pIDisplay);
}

// flag  ȵ..
void drawCut(CMoso* pMe,int resID, int x, int y, int row, int col, int xsize, int ysize)
{
	int xoff,yoff;
	int x1,y1,dx1,dy1;
	AEERect rtBefore;
	
	// save original clip region.
	rtBefore = pMe->clipRect;

	// correction region with clipRect
	//  Ŭ  ִٸ ׷ Ǵ ̶ ļ  ش.
	x1 = ( pMe->clipRect.x > x ) ? pMe->clipRect.x : x;
	dx1 = ( pMe->clipRect.x + pMe->clipRect.dx < x + xsize ) ? pMe->clipRect.x + pMe->clipRect.dx : x + xsize;
	dx1 -= x1;

	y1 = ( pMe->clipRect.y > y ) ? pMe->clipRect.y : y;	
	dy1 = ( pMe->clipRect.y + pMe->clipRect.dy < y + ysize ) ? pMe->clipRect.y + pMe->clipRect.dy : y + ysize;
	dy1 -= y1;

	// ̹ ŭ ̵ ؾ Ǵ ϰ..
	xoff = col*xsize;
	yoff = row*ysize;

	// ׸.
	setClip(pMe,x1,y1,dx1,dy1);
	drawImage(pMe,resID,(int)(x-xoff),(int)(y-yoff),GFX_LEFTTOP);

	//  Ŭ  ְ...
	setClip(pMe,rtBefore.x,rtBefore.y,rtBefore.dx,rtBefore.dy);
}

//  ڿ Ѿ x      ڸ Ѵ.
void drawGNumber(CMoso* pMe,int num,int x,int y,int type)
{
	char	temp[20];
	char	*curr;
	char	one_char[2];
	int		one_digit;
	int		image,dx,dy,gap;
	int		i,len;

	switch( type )
	{
		case GNUM_TYPE1:
			image	= IMG_NUMBER1;
			dx		= pMe->imgGame[IMG_NUMBER1].info.cx/10;
			dy		= pMe->imgGame[IMG_NUMBER1].info.cy;
			gap		= 0;
			break;

		case GNUM_TYPE2:
			image	= IMG_NUMBER2;
			dx		= pMe->imgGame[IMG_NUMBER2].info.cx/10;
			dy		= pMe->imgGame[IMG_NUMBER2].info.cy;
			gap		= 0;
			break;

		case GNUM_TYPE3:
			image	= IMG_NUMBER3;
			dx		= pMe->imgGame[IMG_NUMBER3].info.cx/10;
			dy		= pMe->imgGame[IMG_NUMBER3].info.cy;
			gap		= 0;
			break;
		default:
			image	= IMG_NUMBER2;
			dx		= pMe->imgGame[IMG_NUMBER2].info.cx/10;
			dy		= pMe->imgGame[IMG_NUMBER2].info.cy;
			gap		= 0;
	}


	MEMSET(temp,0,20);
	SPRINTF(temp,"%d",num);
	curr = temp;
	len = STRLEN(temp);

	for(i=len-1;i>-1;i--)
	{
		MEMSET(one_char,0,sizeof(char)*2);
		one_char[0] = *(curr+i);
		
		one_digit = ATOI(one_char);

		drawCut(pMe,image,x-dx,y,0,one_digit,dx,dy); 

		x = x - dx - gap;	// ڵ  5 pixel ε 6ȼ ָ ȴ..
	}
}

void initText(CMoso* pMe,char* sbText,int dx,int dy,int flag)
{
	int sbLen,mbLen;
	int sum_line,sum_width;
	AECHAR mbTemp[2];
	AECHAR* pCurr;
	
	if( pMe->text_init == TRUE || sbText == NULL ) return;



	dx -= TEXT_HORZMARGIN*2;
	dy -= TEXT_VERTMARGIN*2;


	pMe->text_wholepage		= 0;
	pMe->text_currpage		= 0;
	pMe->text_wholeline		= 0;
	pMe->text_vgap			= pMe->font_h + 4;
	pMe->text_lineperpage	= dy / pMe->text_vgap;
	pMe->text_align			= flag;
	
	

	pMe->text_reachEnd = FALSE;


	sum_width	= 0;
	sum_line	= 0;
	
	// move single byte char to multibyte char
	sbLen	= STRLEN(sbText);
	mbLen	= sbLen*2 + 1;
	pMe->text_mbText = MALLOC(mbLen);
	MEMSET(pMe->text_mbText,0,mbLen);	
	StrToWStr(sbText,pMe->text_mbText);


	// we'll get how much pages in text.
	pCurr = pMe->text_mbText;
	while( *pCurr != '\0' )
	{
		sbLen = sizeof(AECHAR);
		MEMSET(mbTemp,0,sizeof(AECHAR)*2);
		MEMCPY(mbTemp,pCurr,sizeof(AECHAR));

		//   ̸..
		if( *pCurr == '\\' || *pCurr == '\0' )
		{
			++pMe->text_wholeline;
			++sum_line;
			sum_width = 0;

			//   Ѿ Ǹ 
			if( sum_line >= pMe->text_lineperpage )
			{
				++pMe->text_wholepage;
				sum_line = 0;
			}
		}
		//  Ѱܾ Ǵ Ȳ̸.
		else if( *pCurr == TEXT_PAGEFEED ||	*(pCurr+1) == '\0' )
		{
			++pMe->text_wholeline;
			++pMe->text_wholepage;
			sum_line = 0;
		}		
		else
		{
			sum_width += IDISPLAY_MeasureText(pMe->a.m_pIDisplay,pMe->font,mbTemp);

			// µǴ ڿ   ѱٸ..
			if( sum_width >= dx - pMe->font_w )
			{
				++pMe->text_wholeline;
				++sum_line;
				sum_width = 0;

				if( sum_line >= pMe->text_lineperpage )
				{
					++pMe->text_wholepage;
					sum_line = 0;
				}
			}
		}

		++pCurr;
	}

	pMe->text_init = TRUE;
	
	return;
}

void drawText(CMoso* pMe,int x,int y,int dx,int dy,boolean showPage)
{
	AECHAR* pCurr;
	AECHAR* pPrev;
	int	copybyte;
	int	page;
	AECHAR	mbTemp[2];
	int	sum_line,sum_width;
	int	posy;

	if( pMe->text_align == GFX_HCENTER )
		x = (x + dx)/2;
	else
		x += TEXT_HORZMARGIN;
	//x	+= TEXT_HORZMARGIN;
	y	+= TEXT_VERTMARGIN;
	dx	-= TEXT_HORZMARGIN*2;
	dy	-= TEXT_VERTMARGIN*2;

	if( pMe->text_Draw == FALSE ) 
	{
		sum_width	= 0;
		sum_line	= 0;
		page		= 0;
		pPrev		= pCurr = pMe->text_mbText;


		if( pMe->text_currpage == pMe->text_wholepage )
			pMe->text_reachEnd = TRUE;

		if(pMe->text_currpage == pMe->text_wholepage)
			--pMe->text_currpage;
		else if(pMe->text_currpage == -1)
			pMe->text_currpage = 0;


		while( *pCurr != '\0' )
		{
			MEMSET(mbTemp,0,sizeof(AECHAR)*2);
			MEMCPY(mbTemp,pCurr,sizeof(AECHAR));


			//   ̸..
			if( *pCurr == '\\' || *pCurr == '\0' )
			{
				posy = y+sum_line*pMe->text_vgap;
				copybyte = pCurr - pPrev;
				MEMSET(pMe->tempWStr,0,100);
				WSTRNCOPYN(pMe->tempWStr,50,pPrev,copybyte);

				// only print current page.
				if( pMe->text_currpage == page &&
					(pMe->clipRect.y <= posy && pMe->clipRect.y + pMe->clipRect.dy >= posy) )
					drawWString(pMe,pMe->tempWStr,x,posy,pMe->text_align);			


				++sum_line;
				sum_width = 0;

				pPrev = pCurr+1;				

				//   Ѿ Ǹ 
				if( sum_line >= pMe->text_lineperpage )
				{
					page++;
					sum_line = 0;
				}
			}
			//  Ѱܾ Ǵ Ȳ̸.
			else if( *pCurr == TEXT_PAGEFEED )
			{
				posy = y+sum_line*pMe->text_vgap;
				copybyte = pCurr - pPrev;
				MEMSET(pMe->tempWStr,0,100);
				WSTRNCOPYN(pMe->tempWStr,50,pPrev,copybyte);

				// only print current page.
				if( pMe->text_currpage == page &&
					(pMe->clipRect.y <= posy && pMe->clipRect.y + pMe->clipRect.dy >= posy) ) 
					drawWString(pMe,pMe->tempWStr,x,posy,pMe->text_align);			
				
				pPrev = pCurr+1;								

				page++;
				sum_line = 0;
			}	
			//   ٸ..
			else if( *(pCurr+1) == '\0' )
			{
				posy = y+sum_line*pMe->text_vgap;
				copybyte = pCurr - pPrev + 1; //  ڸ   Ѵ.
				MEMSET(pMe->tempWStr,0,100);
				WSTRNCOPYN(pMe->tempWStr,50,pPrev,copybyte);

				// only print current page.
				if( pMe->text_currpage == page &&
					(pMe->clipRect.y <= posy && pMe->clipRect.y + pMe->clipRect.dy >= posy) ) 
					drawWString(pMe,pMe->tempWStr,x,posy,pMe->text_align);			

				pPrev = pCurr+1;				

				page++;
				sum_line = 0;
			}
			else
			{
				sum_width += IDISPLAY_MeasureText(pMe->a.m_pIDisplay,pMe->font,mbTemp);

				// µǴ ڿ   ѱٸ..
				if( sum_width >= dx - pMe->font_w )
				{
					posy = y+sum_line*pMe->text_vgap;
					copybyte = pCurr - pPrev;
					MEMSET(pMe->tempWStr,0,100);
					WSTRNCOPYN(pMe->tempWStr,50,pPrev,copybyte);

					// only print current page.
					if( pMe->text_currpage == page )
						drawWString(pMe,pMe->tempWStr,x,posy,pMe->text_align);			

					++sum_line;
					sum_width = 0;

					if( sum_line >= pMe->text_lineperpage )
					{
						page++;
						sum_line = 0;
					}

					pPrev = pCurr;
					while(*pPrev == ' ')	pPrev++;		//  
				}
			}
			++pCurr;
		}


		// show currpage/wholepage.
		if( showPage == TRUE )
		{
			x = x + dx/2;
			y = y + dy - pMe->font_h;
			MEMSET(pMe->tempStr,0,sizeof(pMe->tempStr));
			SPRINTF(pMe->tempStr,"%d/%d",pMe->text_currpage+1,pMe->text_wholepage);
			drawString(pMe,pMe->tempStr,x,y,GFX_TOP|GFX_HCENTER);
		}

		// prevent redraw
		pMe->text_Draw = TRUE;
	}
}

void stopText(CMoso* pMe)
{
	pMe->text_init = FALSE;
	pMe->text_Draw = FALSE;
	pMe->text_reachEnd = FALSE;
	if( pMe->text_mbText )
		FREE(pMe->text_mbText);
}


void initAnitext(CMoso* pMe,char* sbText,int timegap,int dx,int flag)
{
	if( pMe->anitext_init == FALSE )
	{
		pMe->anitext_init = TRUE;

		// 1000    1000 pixel ν  ڵ     ְ ȴ.
		initText(pMe,sbText,dx,5000,flag);

		pMe->anitext_startTime	= GET_UPTIMEMS();
		pMe->anitext_timegap	= timegap;
		pMe->anitext_y			= -pMe->canvas_height/2;
		pMe->anitext_frame		= 0;
	}
}

boolean drawAnitext(CMoso* pMe,int x,int y,int dx,int dy)
{
	int i,j;
	int idx,idy;	// image dx, image dy
	int nx,ny;	// new position

	//  ð ʰϸ..
	if( GET_UPTIMEMS() - pMe->anitext_startTime > pMe->anitext_timegap )
	{
		pMe->anitext_startTime = GET_UPTIMEMS();

		// draw text
		setClip(pMe,x,y,dx,dy);

		//   ٽ ׷ֱ.
		idx = pMe->imgGame[IMG_STATUSBG].info.cx;
		idy = pMe->imgGame[IMG_STATUSBG].info.cy;

		nx = pMe->canvas_width / idx;
		ny = pMe->canvas_height/ idy;

		for(i=0;i<=nx;++i)
			for(j=0;j<=ny;++j)
				drawImage(pMe,IMG_STATUSBG,i*idx,j*idy,GFX_LEFTTOP);

		
		pMe->anitext_y += pMe->font_h;
		pMe->text_Draw = FALSE;

		drawText(pMe,x,y-pMe->anitext_y,dx,dy+pMe->anitext_y,FALSE);

		//   ÷ش.
		pMe->anitext_frame++;
	}
	
	// ũ  Ǿٸ..
	if(	pMe->text_wholeline*pMe->text_vgap < pMe->anitext_y )
	{
		stopAnitext(pMe);
		return TRUE;
	}
	else
		return FALSE;
}


void stopAnitext(CMoso* pMe)
{
	stopText(pMe);
	pMe->anitext_init	= FALSE;
	pMe->anitext_y		= 0;
}




int getRand(CMoso* pMe,int maxNum)
{
	pMe->randi++;
	if( pMe->randi > 199 )
		pMe->randi = 0;

	return (int)(pMe->rand[pMe->randi] % maxNum);	
}


void setGauge(CMoso* pMe,int x,int y,int max,int resId,int resbgId,int d)
{
	int i = pMe->gauge_flag;

	//   ִ   3   .
	if( pMe->gauge_flag > 3 )
		return;

	pMe->gauge_x[i]			= x;
	pMe->gauge_y[i]			= y;
	pMe->gauge_width[i]		= pMe->imgGame[resId].info.cx;
	pMe->gauge_height[i]	= pMe->imgGame[resId].info.cy;
	pMe->gauge_d[i]			= ( d ) ? TRUE : FALSE; // 0 : horz , 1 : vert
	pMe->gauge_max[i]		= max;
	pMe->gauge_resId[i]		= resId;
	pMe->gauge_resbgId[i]	= resbgId;	// -1 : there is no background img.
	

	pMe->gauge_flag++;
}

void drawGauge(CMoso* pMe,int gauge,int seq)
{
	int	i;
	int	x,y,dx,dy;
	int	percent;
	int resId,resbgId;
	boolean	d;	// 0 : horz, 1 : vert	

	// if score is minus, just return
	if( gauge < 0 )	return;

	for(i=0;i<pMe->gauge_flag;i++)
	{
		// 
		if( i == seq )
		{
			x	= pMe->gauge_x[i];
			y	= pMe->gauge_y[i];
			dx	= pMe->gauge_width[i];
			dy	= pMe->gauge_height[i];
			d	= pMe->gauge_d[i];
			resId	= pMe->gauge_resId[i];
			resbgId = pMe->gauge_resbgId[i];
/*
			// if we have read all resources, set percent value to barlen.
			if( pMe->gauge_max[i]-1 <= gauge )
			{
				percent = ( d ) ? dy : dx;
			}
			// get percent how much resources we've read.
			else
			{
				percent = (gauge << 8 / pMe->gauge_max[i];
				percent = ( percent * (( d ) ? dy : dx) ) >> 8;
			}
*/
				percent = (gauge << 8) / pMe->gauge_max[i];
				percent = ( percent * (( d ) ? dy : dx) ) >> 8;


			// draw gauge background
			if( resbgId != -1 )
			{
				setClip(pMe,x,y,dx,dy);
				fillRect(pMe,x,y,dx,dy,RGB(0,0,0));
				drawImage(pMe,resbgId,x,y,GFX_LEFT);
			}

			// draw gauge
			x = pMe->gauge_x[i];
			y = pMe->gauge_y[i];
			dx = pMe->gauge_width[i];
			dy = pMe->gauge_height[i];

			if(d==0)	setClip(pMe,x,y,percent,dy);
			else		setClip(pMe,x,y,dx,percent);

			drawImage(pMe,resId,x,y,GFX_LEFT);
		}
	}
}

void releaseGauge(CMoso* pMe)
{
	pMe->gauge_flag = 0;
}


void LogHeapStatsToFile(CMoso *pMe, int i)
{
	IFileMgr * pIFileMgr;

	if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_FILEMGR, (void**)&pIFileMgr) == SUCCESS)
	{
		IFile * pIFile;
		if (IFILEMGR_Test(pIFileMgr, LOGFILE) != SUCCESS)
		pIFile = IFILEMGR_OpenFile(pIFileMgr, LOGFILE, _OFM_CREATE);
		else
		pIFile = IFILEMGR_OpenFile(pIFileMgr, LOGFILE, _OFM_APPEND);

		if (pIFile != NULL)
		{
			//JulianType jt;
			//char timestamp[30];
			char buf[100];
			//GETJULIANDATE(GETTIMESECONDS(), &jt);
			//SPRINTF(timestamp, "[%02d:%02d:%02d] ", jt.wHour, jt.wMinute, jt.wSecond);
			SPRINTF(buf, ">>> %d. Memory: %d, Used: %d", i, pMe->dm.dwRAM,IHEAP_GetMemStats(pMe->pIHeap));
			//IFILE_Write(pIFile, timestamp, STRLEN(timestamp));
			IFILE_Write(pIFile, buf, STRLEN(buf));
			IFILE_Write(pIFile, "\r\n", 2);
			IFILE_Release(pIFile);
		}
		IFILEMGR_Release(pIFileMgr);
	}
}

