#include "moso.h"
#include "happysnd.h"

void sfx_init(CMoso* pMe) 
{	
#ifdef AEE_SIMULATOR
	return;
#else
	// create ISound.
	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **)&pMe->m_pISound);

	pMe->soundPlayer			= NULL;
	pMe->soundClip				= NULL;
		
	pMe->soundPlaying			= FALSE;

	pMe->soundId				= 0;
	pMe->soundRepeat			= FALSE;
#endif
}

void sfx_free(CMoso* pMe)
{
#ifdef AEE_SIMULATOR
	return;
#else
	sfx_stop(pMe);

	if( pMe->m_pISound ) 
		ISOUND_Release(pMe->m_pISound);
#endif
}

void sfx_pending(CMoso* pMe) {
#ifdef AEE_SIMULATOR
	return;
#else
	sfx_play(pMe,pMe->soundPendingId,pMe->soundPendingRepeat);
#endif
}


void sfx_play(CMoso* pMe, int id, boolean repeat) 
{
#ifdef AEE_SIMULATOR
	return;
#else
	uint32 dwSize;
	byte *buffer;

	if (!pMe->config_isSound) {
		return;
	}

	// if there is already playing mmf, stop it and call sfx_pending()
	if (pMe->soundPlaying)
	{
		sfx_stop(pMe);		

		// set pending mmf
		pMe->soundPendingId		= id;
		pMe->soundPendingRepeat	= repeat;

		ISHELL_SetTimer(pMe->a.m_pIShell, 1, (PFNNOTIFY)(sfx_pending), pMe);
        return;
    }

	// set flag
	pMe->soundPlaying	= TRUE;
	pMe->soundId		= id;
	pMe->soundRepeat	= repeat;
		

	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUNDPLAYER, (void**) &pMe->soundPlayer);
	ISOUNDPLAYER_RegisterNotify(pMe->soundPlayer, &sfx_callback, (void*) pMe);	

	buffer = (byte*)ISHELL_LoadResData(pMe->a.m_pIShell, MOSO_RES_FILE, (uint16)pMe->sndID[pMe->soundId], RESTYPE_IMAGE);


	// get mmf file size
	dwSize = GET_DWORD_FROM_PTR(buffer + RES_HEADER_SIZE + SIZE_INFO_OFFSET) + MMF_HEADER_SIZE;
	pMe->soundClip = MALLOC(dwSize);

	MEMCPY(pMe->soundClip, buffer + RES_HEADER_SIZE, dwSize);
	ISHELL_FreeResData(pMe->a.m_pIShell, buffer);	


	ISOUNDPLAYER_Set(pMe->soundPlayer,  SDT_BUFFER, (void *)pMe->soundClip);
	ISOUNDPLAYER_Play(pMe->soundPlayer);
#endif
}

void sfx_stop(CMoso* pMe) 
{
#ifdef AEE_SIMULATOR
	return;
#else
	if( pMe->soundPlaying == TRUE ) 
	{
		// set flag
		pMe->soundPlaying	= FALSE;
		pMe->soundId		= 0;
		pMe->soundRepeat	= FALSE;

		ISOUNDPLAYER_RegisterNotify(pMe->soundPlayer, NULL, NULL);
		ISOUNDPLAYER_Stop(pMe->soundPlayer);		
	}

	if( pMe->soundPlayer ) {
		ISOUNDPLAYER_Release(pMe->soundPlayer);
		pMe->soundPlayer = NULL;
	}

	if(pMe->soundClip != NULL)
	{
		FREE(pMe->soundClip);
		pMe->soundClip = NULL;
	}
#endif
}

// call this function when pick up the call.
void sfx_suspend(CMoso* pMe)
{
	// some handset keep playing sounds and vibration when having a call.
	// so stop sound and vibration here certainly.
	ISOUND_Get(pMe->m_pISound, &pMe->soundInfo);
	ISOUND_StopTone(pMe->m_pISound);
	ISOUND_StopVibrate(pMe->m_pISound);


	pMe->soundPendingId		= pMe->soundId;
	pMe->soundPendingRepeat	= pMe->soundRepeat;

	sfx_stop(pMe);
}

// call this function after hang up the call.
void sfx_resume(CMoso* pMe)
{
	ISOUND_Set(pMe->m_pISound, &pMe->soundInfo);
	ISOUND_SetDevice(pMe->m_pISound);

	//sfx_play(pMe,pMe->soundPendingId,pMe->soundPendingRepeat);
}

void sfx_vib(CMoso* pMe,int sec)
{
	ISOUND_Vibrate(pMe->m_pISound, (int16)sec); 
}

void sfx_callback(void* pUser, AEESoundPlayerCmd eCBType, AEESoundPlayerStatus eStatus, uint32 dwParam) 
{
#ifdef AEE_SIMULATOR
	return;
#else
	CMoso* pMe = (CMoso*) pUser;
	int id,repeat;

	if (!pMe->config_isSound) {
		return;
	}

	// some handsets have 'AEE_SOUNDPLAYER_DONE' in eStatus and 
	// some handsets have 'AEE_SOUNDPLAYER_PLAY_CB' in eStatus.
	switch (eStatus) 
	{
		case AEE_SOUNDPLAYER_DONE:
			if (eCBType == AEE_SOUNDPLAYER_PLAY_CB) {
				id		= pMe->soundId;
				repeat	= pMe->soundRepeat;

				sfx_stop(pMe);

				// repeat sound
				if( repeat == TRUE ) {
					// set pending mmf
					pMe->soundPendingId		= id;
					pMe->soundPendingRepeat	= repeat;

					ISHELL_SetTimer(pMe->a.m_pIShell, 1, (PFNNOTIFY)(sfx_pending), pMe);
				}
			}
			break;
		case AEE_SOUNDPLAYER_ABORTED:
			if(eCBType == AEE_SOUNDPLAYER_PLAY_CB) {
				id		= pMe->soundId;
				repeat	= pMe->soundRepeat;

				sfx_stop(pMe);

				// repeat sound
				if( repeat == TRUE ) {
					// set pending mmf
					pMe->soundPendingId		= id;
					pMe->soundPendingRepeat	= repeat;

					ISHELL_SetTimer(pMe->a.m_pIShell, 1, (PFNNOTIFY)(sfx_pending), pMe);
				}
			}
			break;
	}
#endif
}
