#include "moso.h"
#include "menu.h"
#include "socket.h"
#include "happysnd.h"

void putRank(CMoso* pMe)
{
	// init sockets
	int iRetValue = AEE_NET_EBADF, lastError;

	initSocket(pMe);
	
#ifdef KTF_BILL_COMM
	// billcom 
	if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_BILLCOM, (void**) &pMe->m_pBillcomMgr) != SUCCESS) {
		ReleaseNetAndSocket(pMe);
		showText(pMe, "can't init bilcom");
		return;
	}

	IEB_KTF_COM_SetNetMgr(pMe->m_pBillcomMgr, pMe->m_pINetMgr, pMe->m_pISocket);
	if (AEE_NET_SUCCESS != (iRetValue = IEB_KTF_COM_Connect(pMe->m_pBillcomMgr, (PFNCONNECTCB) putconnectCB, pMe, FALSE)) ) {
		if (iRetValue != AEE_NET_WOULDBLOCK) {
			showText(pMe, "failed to connect");
			// Get Last error
			lastError = ISOCKET_GetLastError(pMe->m_pISocket);
			// Release NetMgr & ISocket and return.
			ReleaseNetAndSocket(pMe);
			return;
		}
	}
#else
    // Connect to the remote host.
	if ((iRetValue = ISOCKET_Connect (pMe->m_pISocket, pMe->nodeINAddr, AEE_htons(HOST_PUTPORT), 
										putconnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
	{
		// If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
		if (iRetValue != AEE_NET_WOULDBLOCK) {

		}
	}
#endif
}

void getRank(CMoso* pMe)
{
	// init sockets
	int iRetValue = AEE_NET_EBADF, lastError;

	initSocket(pMe);	

#ifdef KTF_BILL_COMM
	// billcom 
	if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_BILLCOM, (void**) &pMe->m_pBillcomMgr) != SUCCESS) {
		ReleaseNetAndSocket(pMe);
		showText(pMe, "can't init bilcom");
		return;
	}

	IEB_KTF_COM_SetNetMgr(pMe->m_pBillcomMgr, pMe->m_pINetMgr, pMe->m_pISocket);
	if (AEE_NET_SUCCESS != (iRetValue = IEB_KTF_COM_Connect(pMe->m_pBillcomMgr, (PFNCONNECTCB) getconnectCB, pMe, FALSE)) ) {
		if (iRetValue != AEE_NET_WOULDBLOCK) {
			showText(pMe, "failed to connect");
			// Get Last error
			lastError = ISOCKET_GetLastError(pMe->m_pISocket);
			// Release NetMgr & ISocket and return.
			ReleaseNetAndSocket(pMe);
			return;
		}
	}
#else
    // Connect to the remote host.
	if ((iRetValue = ISOCKET_Connect (pMe->m_pISocket, pMe->nodeINAddr, AEE_htons(HOST_GETPORT), 
										getconnectCB, (void *)pMe))  != AEE_NET_SUCCESS)
	{
		// If it's WOULDBLOCK, then it's still connecting. Don't cleanup.
		if (iRetValue != AEE_NET_WOULDBLOCK) {

		}
	}
#endif
}

void putconnectCB(CMoso * pMe, int iError) 
{
	SENDSCORE	score;

	// Check if connect failed
	if (iError) 
	{
		SPRINTF(pMe->tempStr, "connect failed - %d", iError);
		showText(pMe, pMe->tempStr);
		// Release the INetMgr and ISocket pointers
		ReleaseNetAndSocket(pMe);
		return;
	}

	// register ranking.
	pMe->sendPos = 0;
	pMe->sendSize = sizeof(SENDSCORE);


	// set sendBuf
	if( pMe->gPrevState == STATE_TRASH )
	{
		score.gameType	= 0;
		score.score1	= pMe->endless_trashlv+1;
		score.score2	= pMe->endless_trashscore;
	}
	else if( pMe->gPrevState == STATE_FIND )
	{
		score.gameType	= 1;
		score.score1	= pMe->endless_findlv+1;
		score.score2	= pMe->endless_findscore;
	}
	else if( pMe->gPrevState == STATE_DIG )
	{
		score.gameType	= 2;
		score.score1	= pMe->endless_diglv+1;
		score.score2	= pMe->endless_digscore;
	}
	MEMSET(pMe->sendBuf,0,sizeof(pMe->sendBuf));
	MEMCPY(pMe->sendBuf,&score,sizeof(SENDSCORE));

	// call callback function.
	putrankCB((void *)pMe);
}

void getconnectCB(CMoso * pMe, int iError) 
{
	SENDTYPE	type;

	// Check if connect failed
	if (iError) 
	{
		SPRINTF(pMe->tempStr, "connect failed - %d", iError);
		showText(pMe, pMe->tempStr);
		// Release the INetMgr and ISocket pointers
		ReleaseNetAndSocket(pMe);
		return;
	}

	// get ranking.
	pMe->sendPos	= 0;
	pMe->sendSize	= sizeof(SENDTYPE);

	// set sendBuf.
	type.gameType	= pMe->menu_RankingSelectCurr;
	MEMSET(pMe->sendBuf,0,sizeof(pMe->sendBuf));
	MEMCPY(pMe->sendBuf,&type,sizeof(SENDTYPE));

	// call callback function.
	getrankCB((void*)pMe);
}

// put rank to server
void putrankCB (CMoso *pMe)
{
	int rv;

#ifdef KTF_BILL_COMM
	rv = IEB_KTF_COM_Write(pMe->m_pBillcomMgr,
							(byte*)(pMe->sendBuf + pMe->sendPos),
							(uint16)(pMe->sendSize - pMe->sendPos),
							pMe->nodeINAddr,
							HOST_PUTPORT);
#else
    rv = ISOCKET_Write(pMe->m_pISocket,
			  (byte*)(pMe->sendBuf + pMe->sendPos),
			  (uint16)(pMe->sendSize - pMe->sendPos));
#endif

	// send rank data from phone to server
	if(rv > 0)
	{
		pMe->sendPos += rv;
		if(pMe->sendSize - pMe->sendPos > 0 )
		{
			ISOCKET_Writeable(pMe->m_pISocket, (PFNNOTIFY)putrankCB, (void*) pMe);
			return;
		}
	}
	else if(rv == AEE_NET_WOULDBLOCK)
	{
		ISOCKET_Writeable(pMe->m_pISocket, (PFNNOTIFY)putrankCB, (void*) pMe);
		return;
	}
	else if(rv == AEE_NET_ERROR || rv <= 0)
	{
		ReleaseNetAndSocket(pMe);
		return;
	}

	// prepare to read macsheader packet
	pMe->readPos = 0;
	pMe->readSize = sizeof(MACSHEADERPKT);

	MEMSET(pMe->readBuf,0,sizeof(pMe->readBuf));

	// call callback function
	ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readmacsCB, (void *)pMe);
}

// get rank from server
void getrankCB (CMoso* pMe)
{
	int	rv;

#ifdef KTF_BILL_COMM
	rv = IEB_KTF_COM_Write(pMe->m_pBillcomMgr,
							(byte*)(pMe->sendBuf + pMe->sendPos),
							(uint16)(pMe->sendSize - pMe->sendPos),
							pMe->nodeINAddr,
							HOST_GETPORT);
#else
    rv = ISOCKET_Write(pMe->m_pISocket,
			  (byte*)(pMe->sendBuf + pMe->sendPos),
			  (uint16)(pMe->sendSize - pMe->sendPos));
#endif

	// send rank data from phone to server
	if(rv > 0)
	{
		pMe->sendPos += rv;
		if(pMe->sendSize > pMe->sendPos)
		{
			ISOCKET_Writeable(pMe->m_pISocket, (PFNNOTIFY)getrankCB, (void*) pMe);
			return;
		}
	}
	else if(rv == AEE_NET_WOULDBLOCK)
	{
		ISOCKET_Writeable(pMe->m_pISocket, (PFNNOTIFY)getrankCB, (void*) pMe);
		return;
	}
	else if(rv == AEE_NET_ERROR || rv <= 0)
	{
		ReleaseNetAndSocket(pMe);
		return;
	}

	// prepare to read macsheader packet
	pMe->readPos = 0;
	pMe->readSize = sizeof(MACSHEADERPKT);

	MEMSET(pMe->readBuf,0,sizeof(pMe->readBuf));

	// call callback function
	ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readmacsCB, (void *)pMe);
}

// read MACSHEADER packet from server
void readmacsCB(CMoso *pMe)
{
	MACSHEADERPKT  Pkt;
	int rv;
	
	SPRINTF(pMe->tempStr,"readmacsCB,%d",pMe->readPos);
	showText(pMe,pMe->tempStr);

	// get macsheader packet
	rv = ISOCKET_Read(
		pMe->m_pISocket,
		(char*)pMe->readBuf + pMe->readPos,
		(pMe->readSize - pMe->readPos)
	);

	if(rv > 0)
	{
		pMe->readPos += rv;
		if(pMe->readSize - pMe->readPos > 0)
		{
			ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readmacsCB, (void*) pMe);
			return;
		}
	}
	else if(rv == AEE_NET_WOULDBLOCK)
	{
		SPRINTF(pMe->tempStr,"AEE_NET_WOULDBLOCK,%d",rv);
		showText(pMe,pMe->tempStr);

		ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readmacsCB, (void*) pMe);
		return;
	}
	else if(rv == AEE_NET_ERROR || rv <= 0)
	{
		SPRINTF(pMe->tempStr,"AEE_NET_ERROR,%d",rv);
		showText(pMe,pMe->tempStr);

		ReleaseNetAndSocket(pMe);		
		return;
	}	

	
	MEMCPY(&Pkt,pMe->readBuf,sizeof(MACSHEADERPKT));
	Pkt.uiLen = AEE_ntohl(Pkt.uiLen);

	// debug
	SPRINTF(pMe->tempStr,"%d,%c%c%c%c",pMe->readSize,Pkt.achSignature2[0],Pkt.achSignature2[1],Pkt.achSignature2[2],Pkt.achSignature2[3]);
	showText(pMe,pMe->tempStr);

	// ް ó
    if(STRNCMP(Pkt.achSignature2, "BCEN", 4) == 0)
    {			
		pMe->readPos = 0;
		pMe->readSize = 256;
		MEMSET(pMe->readBuf,0,sizeof(pMe->readBuf));

		ISOCKET_Cancel(pMe->m_pISocket, (PFNNOTIFY)readmacsCB, (void*) pMe);
		ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)reademgcyCB, (void*)pMe); 
        return;
    }
	//   ƴ .
	else
	{
		if( pMe->ranking_type == 0 ) // ŷ   
		{
			// cancel callback fuction
			ISOCKET_Cancel(pMe->m_pISocket, (PFNNOTIFY)putrankCB, (void*) pMe);

			// set ranking_result
			pMe->ranking_result = 1;
			ReleaseNetAndSocket(pMe); // ŷ Ͻô  Ͱ  ̹   ̹Ƿ  ش.
		}
		else if( pMe->ranking_type == 1 ) 
		{
			// ŷ   ش Ŷ Ŀ  ŷ  ó ؾ ϹǷ 
			//  ݹ Լ ϰ ο ݹ Ѵ.
			pMe->readPos = 0;
			pMe->readSize = Pkt.uiLen;


			MEMSET(pMe->readBuf,0,sizeof(pMe->readBuf));
			MEMSET(pMe->ranking_data,0,sizeof(RANKDATA)*5);
			MEMSET(&pMe->ranking_mydata,0,sizeof(RANKDATA));

			ISOCKET_Cancel(pMe->m_pISocket, (PFNNOTIFY)readmacsCB, (void*) pMe);
			ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readrankCB, (void*) pMe);
		}
	}
}

// read ranking datas from server.
void readrankCB(CMoso* pMe)
{	
	int nRead; // where do we read in readBuf .
	int i,rv;

	showText(pMe,"readrankCB2");	

	rv = ISOCKET_Read(
		pMe->m_pISocket,
		(char*)(pMe->readBuf + pMe->readPos),
		(pMe->readSize - pMe->readPos)
	);

	if(rv > 0)
	{
		pMe->readPos += rv;
		if(pMe->readSize - pMe->readPos > 0)
		{
			ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readrankCB, (void*) pMe);
			return;
		}
	}
	else if(rv == AEE_NET_WOULDBLOCK)
	{
		SPRINTF(pMe->tempStr,"AEE_NET_WOULDBLOCK2,%d",rv);
		showText(pMe,pMe->tempStr);

		ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)readrankCB, (void*) pMe);
		return;
	}
	else if(rv == AEE_NET_ERROR || rv <= 0)
	{
		SPRINTF(pMe->tempStr,"AEE_NET_ERROR2,%d",rv);
		showText(pMe,pMe->tempStr);

		ReleaseNetAndSocket(pMe);
		return;
	}

	nRead = 0;
	
	// find how many top ranking datas have been read. 
	pMe->ranking_dataCnt = (int)*(pMe->readBuf + nRead);
	nRead = sizeof(int);

	// fill top ranking data
	for(i=0;i<pMe->ranking_dataCnt;i++)
	{
		MEMCPY(&pMe->ranking_data[i],pMe->readBuf + nRead,sizeof(RANKDATA));
		pMe->ranking_data[i].rank	= AEE_ntohl(pMe->ranking_data[i].rank);
		pMe->ranking_data[i].score1 = AEE_ntohl(pMe->ranking_data[i].score1);
		pMe->ranking_data[i].score2 = AEE_ntohl(pMe->ranking_data[i].score2);
		nRead += sizeof(RANKDATA);
	}

	// find whether my ranking data is or not.
	pMe->ranking_mydataCnt = (int)*(pMe->readBuf + nRead);
	nRead += sizeof(int);

	// fill my rankdata with reaBuf
	for(i=0;i<pMe->ranking_mydataCnt;i++)
	{
		MEMCPY(&pMe->ranking_mydata,pMe->readBuf + nRead,sizeof(RANKDATA));
		pMe->ranking_mydata.rank	= AEE_ntohl(pMe->ranking_mydata.rank);
		pMe->ranking_mydata.score1	= AEE_ntohl(pMe->ranking_mydata.score1);
		pMe->ranking_mydata.score2	= AEE_ntohl(pMe->ranking_mydata.score2);
		nRead += sizeof(RANKDATA);
	}

	// cancel callback fuction
	ISOCKET_Cancel(pMe->m_pISocket, (PFNNOTIFY)readrankCB, (void*) pMe);

	// set ranking_result
	pMe->ranking_result = 1;
	ReleaseNetAndSocket(pMe);// we've readed all datas from server, so release socket
}

void reademgcyCB(CMoso* pMe)
{
	int rv=0;

	rv = ISOCKET_Read(
		pMe->m_pISocket,
		(char*)pMe->readBuf,
		pMe->readSize - pMe->readPos
	);

	if(rv > 0)
	{
		pMe->readPos += rv;
		if(pMe->readSize - pMe->readPos > 0)
		{
			ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)reademgcyCB, (void*) pMe);
			return;
		}

	}
	else if(rv == AEE_NET_WOULDBLOCK)
	{
		ISOCKET_Readable(pMe->m_pISocket, (PFNNOTIFY)reademgcyCB, (void*) pMe);
		return;
	}
	else if(rv == AEE_NET_ERROR || rv <= 0)
	{
		ReleaseNetAndSocket(pMe);
		return;
	}	

	// transfer result to pMe->ranking_emergency
	MEMCPY(pMe->ranking_emergency,pMe->readBuf,256);

	// we've readed all datas from server, so release socket
	ISOCKET_Cancel(pMe->m_pISocket, (PFNNOTIFY)reademgcyCB, (void*) pMe);

	// set ranking_result
	pMe->ranking_result = 2;
	ReleaseNetAndSocket(pMe);
}

// make connection between phone and billcom(or server)
void initSocket(CMoso* pMe)
{
	// default value is fail. 
	pMe->ranking_result = 0 ;


	// Create INetMgr object. This step needs to be done only if
	// the INetMgr object has not been created already. 
	ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **) &pMe->m_pINetMgr);
	if (pMe->m_pINetMgr == NULL) {
		showText(pMe, "can't create netmgr");
		return;
	}
	
	// Open socket
	pMe->m_pISocket = INETMGR_OpenSocket(pMe->m_pINetMgr, AEE_SOCK_STREAM); 
	if (pMe->m_pISocket == NULL) {
		INETMGR_Release(pMe->m_pINetMgr);
		showText(pMe, "can't open sock");
		return;
	}

	// Before we can pass on the address to connect the socket, 
	// we need to convert the address from a string to the INAddr format.
	pMe->nodeINAddr = ConvertToINAddr(HOST_IP_ADDRESS);

	//LogHeapStatsToFile(pMe, 100);
}

// break off connection
void ReleaseNetAndSocket(CMoso* pMe) 
{
	if(pMe->m_pINetMgr)
	{
		INETMGR_SetLinger(pMe->m_pINetMgr, 0);
	}
	
#ifdef KTF_BILL_COMM
	if(pMe->m_pBillcomMgr)
	{
		IEB_KTF_COM_Release(pMe->m_pBillcomMgr);
		pMe->m_pBillcomMgr = NULL;
	}
#endif
	
	if(pMe->m_pISocket)
	{
		ISOCKET_Cancel(pMe->m_pISocket, NULL, NULL);
		ISOCKET_Release(pMe->m_pISocket);
		pMe->m_pISocket = NULL;
	}
	
	if(pMe->m_pINetMgr)
	{
		INETMGR_Release(pMe->m_pINetMgr);
		pMe->m_pINetMgr = NULL;
	}

	// 
	if( pMe->ranking_result == 0 )
		gotoMenu(pMe,MSTATE_RANKINGERROR);
	// 
	else if( pMe->ranking_result == 1 )
		gotoMenu(pMe,MSTATE_RANKINGRESULT);
	//  
	else if( pMe->ranking_result == 2 )
		gotoMenu(pMe,MSTATE_RANKINGEMERGENCY);

	//LogHeapStatsToFile(pMe, 101);
}



INAddr ConvertToINAddr(char *psz)
{
   INAddr ul = 0;
   int nByte = 0;
   char c;

   if(!psz)
      return 0;

   while (ISDIGIT(*psz)) {
      int n = 0;
      while ( ISDIGIT(c=*psz)) {
         n = n*10 + (c - '0');
         ++psz;
      }
      ((char*)&ul)[nByte++] = n;
 
      if (nByte == 4 || *psz != '.')
         break;
 
      ++psz;
   }
 
   if (nByte < 4 || ISALNUM(*psz))
      ul = 0xFFFFFFFF;    // Invalid address

   return ul;
}



void showText(CMoso* pMe, char* text) 
{
//	setScreen(pMe);
//	IDISPLAY_ClearScreen(pMe->a.m_pIDisplay);
//	drawString(pMe,text,0,pMe->canvas_midy,GFX_LEFT);
}